@@ -23,11 +23,13 @@ cd "${objdir}"/tmp
 cat "${testdir}"/../COPYING > in || framework_failure
 fail=0
 
-"${LZIP}" -cd "${testdir}"/COPYING.lz | cmp "${testdir}"/../COPYING - || fail=1
+"${LZIP}" -cd "${testdir}"/COPYING.lz > copy || fail=1
+cmp in copy || fail=1
 
 for i in 1 2 3 4 5 6 7 8 9; do
 	"${LZIP}" -k -$i in || fail=1
 	mv -f in.lz copy.lz || fail=1
+	echo -n "garbage" >> copy.lz || fail=1
 	"${LZIP}" -df copy.lz || fail=1
 	cmp in copy || fail=1
 	echo -n .
@@ -35,6 +37,7 @@ done
 
 for i in 1 2 3 4 5 6 7 8 9; do
 	"${LZIP}" -c -$i in > out || fail=1
+	echo -n "g" >> out || fail=1
 	"${LZIP}" -cd out > copy || fail=1
 	cmp in copy || fail=1
 	echo -n .
